@@ -1607,7 +1607,7 @@ Perl_do_readline(pTHX)
 	    && ckWARN2(WARN_GLOB, WARN_CLOSED))
 	{
 	    if (type == OP_GLOB)
-		Perl_warner(aTHX_ packWARN(WARN_GLOB),
+		Perl_ck_warner_d(aTHX_ packWARN(WARN_GLOB),
 			    "glob failed (can't start child: %s)",
 			    Strerror(errno));
 	    else
