@@ -7,8 +7,8 @@ afsroot='/afs'
 alignbytes='8'
 ansi2knr=''
 aphostname='/bin/hostname'
-archlib='/usr/local/lib/perl5/5.15/unknown'
-archlibexp='/usr/local/lib/perl5/5.15/unknown'
+archlib='/usr/local/lib/perl5/5.16/unknown'
+archlibexp='/usr/local/lib/perl5/5.16/unknown'
 archname='unknown64'
 asctime_r_proto='0'
 bin='/usr/local/bin'
@@ -224,6 +224,7 @@ d_inetaton='undef'
 d_inetntop='undef'
 d_inetpton='undef'
 d_int64_t='undef'
+d_ipv6_mreq='undef'
 d_isascii='undef'
 d_isblank='undef'
 d_isfinite='undef'
@@ -388,6 +389,7 @@ d_sigsetjmp='undef'
 d_sin6_scope_id='undef'
 d_sitearch='undef'
 d_snprintf='undef'
+d_sockaddr_in6='undef'
 d_sockaddr_sa_len='undef'
 d_sockatmark='undef'
 d_sockatmarkproto='undef'
@@ -678,8 +680,8 @@ package='perl5'
 perl_static_inline='static'
 phostname='hostname'
 pidtype=int
-privlib='/usr/local/lib/perl5/5.15'
-privlibexp='/usr/local/lib/perl5/5.15'
+privlib='/usr/local/lib/perl5/5.16'
+privlibexp='/usr/local/lib/perl5/5.16'
 procselfexe=''
 prototype='undef'
 ptrsize='8'
@@ -729,11 +731,11 @@ sig_name_init='0'
 sig_num_init='0'
 sig_size='1'
 signal_t=int
-sitearch='/usr/local/lib/perl5/5.15/unknown'
-sitearchexp='/usr/local/lib/perl5/5.15/unknown'
-sitelib='/usr/local/lib/perl5/5.15'
+sitearch='/usr/local/lib/perl5/5.16/unknown'
+sitearchexp='/usr/local/lib/perl5/5.16/unknown'
+sitelib='/usr/local/lib/perl5/5.16'
 sitelib_stem='/usr/local/lib/perl5'
-sitelibexp='/usr/local/lib/perl5/5.15'
+sitelibexp='/usr/local/lib/perl5/5.16'
 sizesize=4
 sizetype='size_t'
 socksizetype='int'
